Title: Resonant Impulses
Author: Steven Yi
Date: 2018.08.08

Description:

A study of class synthesis technique of using an impulse generator with resonant filter.  Implemented using Csound with:

1. gbuzz opcode for bandlimited impulses
2. zdf_2pole for band-pass filter with high Q setting; filter center frequency swept with exponential envelope
3. Additional declicking and exponential envelope use to shape gesture
4. reverbsc opcode used as always-on reverb effect
5. Temporal Recursion Player instrument used to randomly initiate gesture playback with various, slightly randomized parameters

<CsoundSynthesizer>
<CsOptions>
-odac --port=10000
</CsOptions>
<CsInstruments>

sr	=	48000
ksmps	= 32	
nchnls	=	2
0dbfs	=	1


/* UTILITY CODE FROM csound-live-code */

/** Utility opcode for declicking an audio signal. Should only be used in instruments that have positive p3 duration. */
opcode declick, a, a
  ain xin
  aenv = linseg:a(0, 0.01, 1, p3 - 0.02, 1, 0.01, 0, 0.01, 0)
  xout ain * aenv
endop

/** Returns random item from karray. */
opcode rand, i, k[]
  kvals[] xin
  indx = int(random(0, lenarray(kvals)))
  ival = i(kvals, indx)
  xout ival
endop

;; Stereo Audio Bus

ga_sbus[] init 16, 2

/** Write two audio signals into stereo bus at given index */
opcode sbus_write, 0,iaa
  ibus, al, ar xin
  ga_sbus[ibus][0] = al
  ga_sbus[ibus][1] = ar
endop

/** Mix two audio signals into stereo bus at given index */
opcode sbus_mix, 0,iaa
  ibus, al, ar xin
  ga_sbus[ibus][0] = ga_sbus[ibus][0] + al
  ga_sbus[ibus][1] = ga_sbus[ibus][1] + ar
endop

/** Clear audio signals from bus channel */
opcode sbus_clear, 0, i
  ibus xin
  aclear init 0
  ga_sbus[ibus][0] = aclear
  ga_sbus[ibus][1] = aclear
endop

/** Read audio signals from bus channel */
opcode sbus_read, aa, i
  ibus xin
  aclear init 0
  al = ga_sbus[ibus][0] 
  ar = ga_sbus[ibus][1] 
  xout al, ar
endop


/* PROJECT CODE */

;; cosine ftable for gbuzz
gicos = ftgen(0, 0, 16384, 11, 1) 

/* Resonant Impulse Sound Generator
  1. Uses gbuzz to produce band-limited impulses 
  2. impulses are fed through a band-pass filter with high Q
  3. Filter center frequency is swept from p6 to p7
  4. Pan is random for this project
  5. 0.3 sent to reverb
*/
instr Gesture1
  ifreq = p4
  iamp = p5
  istart_center = p6
  iend_center = p7

  asig = gbuzz(iamp, ifreq, 128, 1, 1, gicos)
  
  /*irange = 14000 - p4*/
;   acenter =  (oscili(0.5, 1) + 0.5) * irange + p4
  acenter = expon:a(istart_center, p3, iend_center)
  asig = zdf_2pole(asig, acenter, 23, 2) ;; Q=23, mode=2 (Band Pass)

  asig *= expon(1, p3, 0.001)
  asig = declick(asig)
 
  
  al, ar pan2 asig, random:i(0.1, 0.9) ;; random pan

  out(al, ar)
  sbus_mix(0, al * 0.3, ar * 0.3)
endin

instr Mix
  al, ar sbus_read 0
  
  al, ar reverbsc al, ar, 0.75, 2200
  
  out(al, ar)
  
  sbus_clear(0)
endin
schedule("Mix", 0, -1)

instr PlayGesture 
  irand = int(random(0,4))
  
  if (irand == 0) then
  	schedule("Gesture1", 0, 4, 15, ampdbfs(-6), 14000, 200)
  elseif (irand == 1) then
  	schedule("Gesture1", 0, 10, 18, ampdbfs(-6), 14000, 4000)
  elseif (irand == 2) then
    schedule("Gesture1", 0, 23, 11.5, ampdbfs(-6), random(100, 200), random(5000, 6000))
  else
    schedule("Gesture1", 0, 23, random(2, 4), ampdbfs(-6), random(100, 200), random(5000, 6000))
  endif

endin

instr Player
  schedule("PlayGesture", 0, 0)

  if (p2 < 120) then
    schedule(p1, rand(array(1,1,1,2,2,6,12)) * random(0.4, 1), 0)
  else
    event_i("e", 0, 28)
  endif
endin

seed(0)
schedule("Player", 0, 0)
</CsInstruments>
</CsoundSynthesizer>

